--
-- PoettingerJumbo10000
-- Specialization for PoettingerJumbo10000 functionality
--
-- @author  	Manuel Leithner (SFM-Modding)
-- @version 	v1.0
-- @date  		30/09/10
--

PoettingerJumbo10000 = {};

function PoettingerJumbo10000.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(TowBall, specializations);
end;

function PoettingerJumbo10000:load(xmlFile)

	self.setHydraulicDirection = SpecializationUtil.callSpecializationsFunction("setHydraulicDirection");

	local hydraulicsCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.hydraulics#count"), 0);	
	self.hydraulics = {};	
	for i=1, hydraulicsCount do
		local hydraulicName = string.format("vehicle.hydraulics.hydraulic%d", i);		
		self.hydraulics[i] = {};		
		self.hydraulics[i].node = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#index"));
		self.hydraulics[i].punch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punch"));
		self.hydraulics[i].translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punchFixpoint"));
		self.hydraulics[i].fixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#fixpoint"));	
		local ax, ay, az;		
		if self.hydraulics[i].punch ~= nil then
			ax, ay, az = getWorldTranslation(self.hydraulics[i].punch);
		else
			ax, ay, az = getWorldTranslation(self.hydraulics[i].node);
		end;
		if self.hydraulics[i].translationPunch ~= nil then
			local bx, by, bz = getWorldTranslation(self.hydraulics[i].translationPunch);		
			self.hydraulics[i].punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
		end;
	end;
	
	self.leftWheelSteeringIndex = 5;
	self.rightWheelSteeringIndex = 4;
	
	self.leftGuideRoller = {};
	self.leftGuideRoller.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.steeringTools#leftGuideRoller"));
	local ax, ay, az = getWorldTranslation(self.leftGuideRoller.node);
	local bx, by, bz = getWorldTranslation(self.hydraulics[5].node);
	local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
	self.leftGuideRoller.size = distance;
	local fixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.steeringTools#fixPoint"));
	local cx,cy,cz = getWorldTranslation(fixPoint);
	self.leftGuideRoller.distance = Utils.vector3Length(cx-bx, cy-by, cz-bz);
	self.rightGuideRoller = {};
	self.rightGuideRoller.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.steeringTools#rightGuideRoller"));
	ax, ay, az = getWorldTranslation(self.rightGuideRoller.node);
	bx, by, bz = getWorldTranslation(self.hydraulics[4].node);
	self.leftGuideRoller.size = Utils.vector3Length(ax-bx, ay-by, az-bz);
	
	self.hayBase = {};
	self.hayBase.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.hayBase#index"));
	self.hayBase.unloadingSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.hayBase#unloadingSpeed"), 0.001);
	self.hayBase.loadingSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.hayBase#loadingSpeed"), 0.0005);
	self.hayBase.speed = self.hayBase.loadingSpeed;
	self.hayBase.unloadingDirection = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.hayBase#unloadingDirection"), 1);
	setShaderParameter(self.hayBase.node, "movingDirection", 0,0,0,0,false);
	
	self.setHydraulicTime = 30;
	self.doTractorFixPointSearch = true;
end;

function PoettingerJumbo10000:delete()
end;

function PoettingerJumbo10000:readStream(streamId, connection)
end;

function PoettingerJumbo10000:writeStream(streamId, connection)
end;

function PoettingerJumbo10000:readUpdateStream(streamId, timestamp, connection)
end;

function PoettingerJumbo10000:writeUpdateStream(streamId, connection, dirtyMask)
end;

function PoettingerJumbo10000:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PoettingerJumbo10000:keyEvent(unicode, sym, modifier, isDown)
end;

function PoettingerJumbo10000:update(dt)	

	if self:getIsActive() then
		local x1,y1,z1 = localDirectionToWorld(self.attacherVehicle.rootNode, 0, 0, 1)
		local x2,y2,z2 = localDirectionToWorld(self.rootNode, 0, 0, 1)
		local dot = x1*x2 + z1*z2;
		dot = dot / (Utils.vector2Length(x1,z1) * Utils.vector2Length(x2,z2))
		angle = math.acos(dot);
		local cy = z1 * x2 - x1 * z2;		
		if cy < 0 then
			angle = -angle;
		end;
		backAngle  = 24 * (angle / 50);
		setRotation(self.leftGuideRoller.node, -backAngle, 0,0);
		setRotation(self.rightGuideRoller.node, backAngle, 0,0);
	end;	
	
	if self:getIsActiveForInput() then
        if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
			if self.isTurnedOn then			
				setShaderParameter(self.hayBase.node, "movingDirection", self.hayBase.unloadingDirection*self.hayBase.loadingSpeed,0,0,0,false);
			else
				setShaderParameter(self.hayBase.node, "movingDirection", 0,0,0,0,false);
			end;
        end;
    end;
	
	if self:getIsActive() then	
		if self.fillLevel >= self.capacity then
			self.isTurnedOn = false;
			setShaderParameter(self.hayBase.node, "movingDirection", 0,0,0,0,false);
		end;
	
		self.setHydraulicTime = 30;	
		
		if self.doTractorFixPointSearch  and self.leftFixPoint ~= nil and self.rightFixPoint ~= nil then
			self.hydraulics[self.leftWheelSteeringIndex].fixPoint = self.leftFixPoint;
			self.hydraulics[self.rightWheelSteeringIndex].fixPoint = self.rightFixPoint;
			self.doTractorFixPointSearch  = false;
		end;
	end;
	
	if self.setHydraulicTime > 0 then
		for i=2, 5 do 
			self:setHydraulicDirection(i);
		end;
		self.setHydraulicTime = self.setHydraulicTime - 1;
	end;
end;

function PoettingerJumbo10000:updateTick(dt)	

end;

function PoettingerJumbo10000:draw()	
end;

function PoettingerJumbo10000:setHydraulicDirection(index)
	local hydraulic = self.hydraulics[index];

	if hydraulic.fixPoint ~= nil then
		local ax, ay, az = getWorldTranslation(hydraulic.node);
		local bx, by, bz = getWorldTranslation(hydraulic.fixPoint);
		local x, y, z = worldDirectionToLocal(getParent(hydraulic.node), bx-ax, by-ay, bz-az);
		
		setDirection(hydraulic.node, x, y, z, 0, 1, 0);
		if hydraulic.punch ~= nil then
			local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
			setTranslation(hydraulic.punch, 0, 0, distance-hydraulic.punchDistance);
		end;
	end;
end;


function PoettingerJumbo10000:onAttach(attacherVehicle)
	self.doJointSearch = true;
end;

function PoettingerJumbo10000:onDetach()
	self.vehicleJoint = nil;
	self.doTractorFixPointSearch = true;
	self.hydraulics[self.leftWheelSteeringIndex].fixPoint = nil;
	self.hydraulics[self.rightWheelSteeringIndex].fixPoint = nil;
	PoettingerJumbo10000.onDeactivate(self);
end;

function PoettingerJumbo10000:onDeactivate()
	setShaderParameter(self.hayBase.node, "movingDirection", 0,0,0,0,false);
end;

function PoettingerJumbo10000:onStartTip(currentTipTrigger, noEventSend)
	self.isTurnedOn = true;
	setShaderParameter(self.hayBase.node, "movingDirection", self.hayBase.unloadingDirection*self.hayBase.unloadingSpeed,0,0,0,false);
end;

function PoettingerJumbo10000:onEndTip(noEventSend)
	self.isTurnedOn = false;
	setShaderParameter(self.hayBase.node, "movingDirection", 0,0,0,0,false);
end;